/*Trabalho 2*/
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <time.h>
#include <string.h>
#include <assert.h>

#include <iostream>
#include <fstream>
#include <stack>
#include <queue>
#include <set>
#include <algorithm>

#define VERBOSE__ 1

using namespace std;

//----
// a structure to represent a weighted edge in graph
struct Edge
{
    int src, dest, weight;
};

// a structure to represent a connected, undirected and weighted graph
struct Graph
{
    // V-> Number of vertices, E-> Number of edges
    int V, E;

    // graph is represented as an array of edges. Since the graph is
    // undirected, the edge from src to dest is also edge from dest
    // to src. Both are counted as 1 edge here.
    struct Edge* edge;
};


// A structure to represent a subset for union-find
struct subset
{
    int parent;
    int rank;
    int tam;
};

struct subproblem {
  int mandatory[8192];
  int disallowed[8192];
  struct subset subsets[81];
  int next_edge;
  int partial_cost;
};

  
#include "CPUTimer.h"

#if (VERBOSE__ == 1)
#define debug_printf(...) fprintf(stderr, __VA_ARGS__)
#endif

#if (VERBOSE__ == 0)
#define debug_printf(...) 
#endif 

// Creates a graph with V vertices and E edges
struct Graph* createGraph(const char* nomeArquivo)
{
	ifstream arquivo(nomeArquivo, ifstream::in);
	if (!arquivo.is_open()) {
		cout << "N�o foi poss�vel abrir \"" << nomeArquivo << "\"." << endl;
		return NULL;
	}

	int n = 0;
	if (arquivo.eof()) {
		cout << "O arquivo \"" << nomeArquivo << "\" est� vazio." << endl;
		arquivo.close();
		return NULL;
	} else {
		arquivo >> n;
	}

    struct Graph* graph = (struct Graph*) malloc( sizeof( struct Graph ) );

    graph->V = n + 1;
    graph->E = (n + 1) * (n + 1);

    graph->edge = (struct Edge*) malloc( graph->E * sizeof( struct Edge ) );

    int valor;
    int k = 0;
	for (int i = 0; i < graph->V; i++) {
		for (int j = 0; j < graph->V; j++) {
			arquivo >> valor;
			if( k < graph->E ) {
				if (arquivo.eof()) {
					cout << "O arquivo \"" << nomeArquivo << "\" est� incompleto." << endl;
					arquivo.close();
					return NULL;
					//i = n + 1;
					//j = n + 1;
				}
				else {
					graph->edge[k].src = i;
					graph->edge[k].dest = j;
					graph->edge[k].weight = valor;
					k++;
				}
			}
		}
	}
	//cout << "Primeira: " << graph->edge[0].weight;
	//cout << " - Ultima: " << graph->edge[--k].weight << endl;
	arquivo.close();

    return graph;
}


// A utility function to find set of an element i
// (uses path compression technique)
int find(struct subset subsets[], int i)
{
    // find root and make root as parent of i (path compression)
    if (subsets[i].parent != i)
        subsets[i].parent = find(subsets, subsets[i].parent);

    return subsets[i].parent;
}

// A function that does union of two sets of x and y
// (uses union by rank)
void Union(struct subset subsets[], int x, int y)
{
    int xroot = find(subsets, x);
    int yroot = find(subsets, y);

    // Attach smaller rank tree under root of high rank tree
    // (Union by Rank)
    if (subsets[xroot].rank < subsets[yroot].rank)
    {
        subsets[xroot].parent = yroot;
    }
    else if (subsets[xroot].rank > subsets[yroot].rank)
    {
        subsets[yroot].parent = xroot;
    }

    // If ranks are same, then make one as root and increment
    // its rank by one
    else
    {
        subsets[yroot].parent = xroot;
        subsets[xroot].rank++;
    }
}

// Compare two edges according to their weights.
// Used in sort() for sorting an array of edges
bool operator<(const struct Edge& a, const struct Edge& b)
{
  return a.weight < b.weight;
}

// The main function to construct MST using Kruskal's algorithm
int KruskalMST(struct Graph* graph, struct Edge result[], int e, subproblem *s, int c)
{
    int V = graph->V;

    // int e = 0;  // An index variable, used for result[]
    int i = 0;  // An index variable, used for sorted edges

    // Number of edges to be taken is equal to V-1
    while (e < V - 1)
    {
        // Step 2: Pick the smallest edge. And increment the index
        // for next iteration
        struct Edge next_edge = graph->edge[i++];
	if (s->disallowed[i - 1] == 0)
	{
	    int x = find(s->subsets, next_edge.src);
	    int y = find(s->subsets, next_edge.dest);
	    
	    // If including this edge does't cause cycle and respect capacity, include it
	    // in result and increment the index of result for next edge
	    if (x != y)
	    {
		result[e++] = next_edge;
		Union(s->subsets, x, y);
		// cout << "Aresta " << (next_edge.src+1) << "-" << (next_edge.dest+1) << " escolhida\n";
	    }
	    // Else discard the next_edge
	}
    }
	
    // print the contents of result[] to display the built MST
    //printf("Following are the edges in the constructed MST\n");
    int total = 0;
    for (i = 0; i < e; ++i) {
	//printf("%d -- %d == %d\n", result[i].src + 1, result[i].dest + 1, result[i].weight);
        total += result[i].weight;
    }
    //printf("MST total cost is %d\n", total);
    
//    delete subsets;
    return total;
}

bool solucao_viavel(struct Graph* graph, struct Edge result[], int Cap)
{
    int Raiz = graph->V - 1;
    stack<int> nos;

    int demanda = 0;
    int v = -1;

    for (int i = 0; i < graph->V; i++)
    {
	if (result[i].src == Raiz)
	{
	    nos.push(result[i].dest);
	    demanda++;
	}
    }

    while(!nos.empty()) 
    {
	v = nos.top();
	nos.pop();
	demanda++;
	if (demanda > Cap)
	{
	    debug_printf("capacidade estourou! demanda = %d, cap = %d\n", demanda, Cap);
	    return false;
	}
	else
	{
	    for(int j = 0; j < graph->V; j++) {
		if (result[j].src == v)
		{
		    nos.push(result[j].dest);
		}
	    }
	}
    }
    return true;
}


int root_connect(struct Graph* graph)
{
	struct Edge next_edge;
	struct Edge* viavel = (struct Edge*) malloc( graph->V * sizeof(struct Edge) );
	int total = 0, i = 0, j = 0;

	while( j < graph->V - 1)
	{
		next_edge = graph->edge[i];
		if( next_edge.dest == graph->V - 1 )
		{
			viavel[j] = next_edge;
			total += next_edge.weight;
			j++;
		}
		i++;
	}

	//for( i = 0; i < graph->V - 1; i++ )
	//	printf("%d -- %d == %d\n", viavel[i].src + 1, viavel[i].dest + 1, viavel[i].weight);

	delete viavel;
	return total;
}

// Driver program to test above functions
int main( int argc, char **argv )
{
    if (argc < 3) {
    	cout << "Sintaxe: t2 <capacidade> <nome do arquivo>" << endl;
    	return EXIT_FAILURE;
    }

    int capacity = atoi(argv[1]);
    struct Graph* graph = createGraph(argv[2]);

    // This will store the resultant MST
    struct Edge* result = (struct Edge*) malloc( graph->V * sizeof(struct Edge) );

    //cout << "Tamanho da inst�ncia: " << graph->V << endl;

    sort(graph->edge, graph->edge + graph->E);

    struct subproblem s0, s1;

    //struct Edge next_edge = graph->edge[0];
    s0.disallowed[0] = 1;
    s1.mandatory[0] = 1;
    s0.next_edge = 1;
    s1.next_edge = 1;

    // Create V subsets with single elements
    for (int j = 0; j < graph->V + 1; ++j)
    {
    	s0.subsets[j].parent = j;
    	s0.subsets[j].rank = 0;
    }

    int lower_bound = KruskalMST(graph, result, 0, &s0, capacity);
    int upper_bound = root_connect(graph); //TODO
    fprintf(stderr, "lower bound = %d, upper bound = %d\n", lower_bound, upper_bound);

    stack<subproblem> subproblems;
    subproblems.push(s0);
    subproblems.push(s1);
    
    CPUTimer totaltime;
    totaltime.reset();

    while(subproblems.size() > 0 && totaltime.getCPUTotalSecs() < 3600)
    {
	totaltime.start();
			
		subproblem s = subproblems.top();
		subproblems.pop();
		int edge_index = 0;
		bool forma_ciclo = false;

		for (int i = 0; i < graph->E; ++i) {
			if (s.mandatory[i] == 1) {
				int x = find(s.subsets, graph->edge[i].src);
				int y = find(s.subsets, graph->edge[i].dest);
				if (x != y)
				{
					result[edge_index] = graph->edge[i];
					edge_index++;
					Union(s.subsets, x, y);
				} else {
					forma_ciclo = true;
					break;
				}
			}
		}

		if (!forma_ciclo) {
			int total_cost = KruskalMST(graph, result, edge_index, &s, capacity);

			if (total_cost < upper_bound) {
              			if (solucao_viavel(graph, result, capacity)) {
				    upper_bound = total_cost;
				}

				subproblem filho1;
				memcpy(filho1.disallowed, s.disallowed, 8192);
				memcpy(filho1.mandatory, s.mandatory, 8192);
				memcpy(filho1.subsets, s.subsets, 81);

				filho1.mandatory[s.next_edge] = 1;
				s.disallowed[s.next_edge] = 1;

				s.next_edge = s.next_edge + 1;
				filho1.next_edge = s.next_edge + 1;

				subproblems.push(s);
				subproblems.push(filho1);
			}
		}

		totaltime.stop();
    }

    printf("Resultado obtido = %d\n", upper_bound);

    delete graph;
    delete result;
    return EXIT_SUCCESS;
}
